"""Command Line Interface for PyAnsys Templates."""

import os

import click

from ansys.templates import AVAILABLE_TEMPLATES_AND_DESCRIPTION, __version__
from ansys.templates.paths import TEMPLATE_PATH_FINDER
from ansys.templates.utils import bake_template


def create_project(template):
    """Create Python project based on a given template.

    Parameters
    ----------
    template : str
        Name of the template to be used as basis for the project

    """
    bake_template(TEMPLATE_PATH_FINDER[template], os.getcwd(), overwrite_if_exists=True)


@click.group()
def main():
    """Ansys tool for creating new Ansys projects."""
    pass


@main.command()
def list():
    """List all available templates names."""
    print("Available templates in ansys-templates are:\n")
    for template_name, description in AVAILABLE_TEMPLATES_AND_DESCRIPTION.items():
        print(f"{template_name.replace('_', '-')}: {description}")


@main.command()
def version():
    """Display current version."""
    print(f"ansys-templates {__version__}")


@main.group()
def new():
    """Create a new project from desired template."""
    pass


@new.command()
def pybasic():
    """Create a basic Python Package."""
    create_project("pybasic")


@new.command()
def pyansys():
    """Create a PyAnsys Python Package project."""
    create_project("pyansys")


@new.command()
def pyansys_advanced():
    """Create an advanced PyAnsys Python Package project."""
    create_project("pyansys_advanced")


@new.command()
def pyace():
    """Create a Python project for any method developers."""
    create_project("pyace")


@new.command()
def pyace_fast():
    """Create a FastAPI project initialized for any developer."""
    create_project("pyace-fast")


@new.command()
def pyace_flask():
    """Create a Flask project initialized for any developer."""
    create_project("pyace-flask")


@new.command()
def pyace_grpc():
    """Create gRPC project initialized for any developer."""
    create_project("pyace-grpc")



from ansys.templates.utils import keep_files


DESIRED_STRUCTURE = [
    ".coveragerc",
    "CHANGELOG.md",
    "CODE_OF_CONDUCT.md",
    "CONTRIBUTING.md",
    "doc/Makefile",
    "doc/make.bat",
    "doc/source/conf.py",
    "doc/source/index.rst",
    "doc/source/_static/README.md",
    "doc/source/_templates/sidebar-nav-bs.html",
    "doc/source/_templates/README.md",
    "examples/README.md",
    ".flake8",
    ".gitignore",
    "LICENSE",
    "pyproject.toml",
    "README.rst",
    "requirements_build.txt",
    "requirements_doc.txt",
    "requirements_tests.txt",
    "setup.py",
    "src/{{ cookiecutter.__project_name_slug }}/__init__.py",
    "tests/test_metadata.py",
]
"""A list holding all desired files to be included in the project."""


def main():
    """Entry point of the script."""
    # Apply the desired structure to the project
    keep_files(DESIRED_STRUCTURE)

if __name__ == "__main__":
    main()



"""A collection of useful paths pointing towards each available template."""

import os
from pathlib import Path

_PATHS_MODULE = Path(os.path.dirname(os.path.abspath(__file__)))

LICENSES_TEMPLATES_PATH = _PATHS_MODULE / "licenses"
"""Path to the software licenses templates."""

PYTHON_TEMPLATES_PATH = _PATHS_MODULE / "python"
"""Path to the Python templates."""

PYTHON_TEMPLATES_COMMON_PATH = PYTHON_TEMPLATES_PATH / "common"
"""Path to the Python common template."""

PYTHON_TEMPLATES_PYBASIC_PATH = PYTHON_TEMPLATES_PATH / "pybasic"
"""Path to the basic Python Package template."""

PYTHON_TEMPLATES_PYANSYS_PATH = PYTHON_TEMPLATES_PATH / "pyansys"
"""Path to the basic PyAnsys Python Package template."""

PYTHON_TEMPLATES_PYANSYS_ADVANCED_PATH = PYTHON_TEMPLATES_PATH / "pyansys_advanced"
"""Path to the advanced PyAnsys Python Package template."""

PYTHON_TEMPLATES_PYACE_PATH = PYTHON_TEMPLATES_PATH / "pyace_pkg"
"""Path to the classic Python Project template."""

PYTHON_TEMPLATES_PYACE_GRPC_PATH = PYTHON_TEMPLATES_PATH / "pyace_grpc"
"""Path to the gRPC based Python Project template."""

PYTHON_TEMPLATES_PYACE_FLASK_PATH = PYTHON_TEMPLATES_PATH / "pyace_flask"
"""Path to the Flask based Python Project template."""

PYTHON_TEMPLATES_PYACE_FAST_PATH = PYTHON_TEMPLATES_PATH / "pyace_fastapi"
"""Path to the FastAPI based Python Project template."""

TEMPLATE_PATH_FINDER = {
    "pybasic": PYTHON_TEMPLATES_PYBASIC_PATH,
    "pyansys": PYTHON_TEMPLATES_PYANSYS_PATH,
    "pyansys_advanced": PYTHON_TEMPLATES_PYANSYS_ADVANCED_PATH,
    "pyace": PYTHON_TEMPLATES_PYACE_PATH,
    "pyace-grpc": PYTHON_TEMPLATES_PYACE_GRPC_PATH,
    "pyace-flask": PYTHON_TEMPLATES_PYACE_FLASK_PATH,
    "pyace-fast": PYTHON_TEMPLATES_PYACE_FAST_PATH,
}
"""A dictionary relating templates names with their paths."""



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}
import pytest

from src.stubs.pingserver_pb2 import EmptyRequest, UserRequest


@pytest.mark.smoke
class TestServer:
    def test_first_ping(self, grpc_stub):
        request = EmptyRequest()
        response = grpc_stub.PingServer(request)
        assert response.message == "Hello, the server is healthy and it had been pinged 1 times!"

    def test_who_ping(self, grpc_stub):
        request = UserRequest(name="you")
        response = grpc_stub.WhoPing(request)
        assert response.message == f"Hello, the server is pinged by {request.name}!"

    def test_third_ping(self, grpc_stub):
        request = EmptyRequest()
        response = grpc_stub.PingServer(request)
        assert response.message == "Hello, the server is healthy and it had been pinged 3 times!"



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}
"""{{ cookiecutter.__project_name_slug }}."""
from ._version import __version__



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}
"""
{{ cookiecutter.project_name }}.

{{ cookiecutter.library_name }}
"""

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__.replace(".", "-"))



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}

"""
{{ cookiecutter.project_name }}.

{{ cookiecutter.library_name }}
"""
from concurrent import futures

import grpc

from observability.logger import Logger
from services.pinger import Pinger
import stubs.pingserver_pb2_grpc


def serve():
    """Serve function."""
    logger = Logger.init(__name__)
    port = 50051
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    stubs.pingserver_pb2_grpc.add_PingerServicer_to_server(Pinger(), server)
    logger.info(f"Server starting and listening on {port}")
    server.add_insecure_port(f"[::]:{port}")
    server.start()
    server.wait_for_termination()


if __name__ == "__main__":
    serve()



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}
from fastapi.testclient import TestClient

from src._version import __version__
from src.server import app


class TestServer:
    def setup_class(self):
        self.client = TestClient(app)

    def test_get_version(self):
        response = self.client.get("/version")
        assert response.status_code == 200
        assert response.json() == {"version": __version__}

    def test_get_health_status(self):
        response = self.client.get("/health")
        assert response.status_code == 200



"""
PyAnsys Templates.

A collection of interactive templates for building Python projects from scratch
according to PyAnsys guidelines.
"""

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__.replace(".", "-"))

AVAILABLE_TEMPLATES_AND_DESCRIPTION = {
    "pybasic": "Create a basic Python Package.",
    "pyansys": "Create a PyAnsys Python Package project.",
    "pyansys_advanced": "Create an advanced PyAnsys Python Package project.",
    "pyace": "Create a Python project for any method developers.",
    "pyace-flask": "Create a Flask project initialized for any developer.",
    "pyace-grpc": "Create gRPC project initialized for any developer.",
    "pyace-fast": "Create a FastAPI project initialized for any developer.",
}
"""A list holding all available templates names."""



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}

"""
{{ cookiecutter.project_name }}.

{{ cookiecutter.library_name }}
"""
import pingserver_pb2
import pingserver_pb2_grpc

NumberPing = 0


class Pinger(pingserver_pb2_grpc.PingerServicer):
    """Pinger class."""

    def PingServer(self, request, context):
        """Ping the server."""
        global NumberPing
        NumberPing += 1
        return pingserver_pb2.PingReply(
            message=f"Hello, the server is healthy and it had been pinged {NumberPing} times!"
        )

    def WhoPing(self, request, context):
        """Check who ping the server."""
        global NumberPing
        NumberPing += 1
        return pingserver_pb2.PingReply(message=f"Hello, the server is pinged by {request.name}!")



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}

"""
{{ cookiecutter.project_name }}.

{{ cookiecutter.library_name }}
"""
from fastapi import FastAPI

from _version import __version__
from observability.logger import Logger

logger = Logger.init("{{ cookiecutter.__project_name_slug }}")

tags_metadata = [
    {
        "name": "{{ cookiecutter.library_name }}",
        "description": "{{ cookiecutter.short_description }}",
    },
]
app = FastAPI(
    title="{{ cookiecutter.project_name }} {{ cookiecutter.library_name }} Application",
    openapi_tags=tags_metadata,
    debug=False,
)


@app.get("/health")
async def health():
    """Check integrity of the server."""
    return "The {{ cookiecutter.library_name }} API server is healthy."


@app.get("/version")
async def version():
    """Check current version."""
    return {"version": __version__}



"""Post-processing script for cleaning the raw rendered project."""
import os
import shutil
from pathlib import Path

import isort

from ansys.templates.utils import keep_files

DESIRED_STRUCTURE = [
    "CHANGELOG.md",
    "CODE_OF_CONDUCT.md",
    "CONTRIBUTING.md",
    "doc/Makefile",
    "doc/make.bat",
    "doc/source/conf.py",
    "doc/source/index.rst",
    "doc/source/_static/README.md",
    "doc/source/_templates/sidebar-nav-bs.html",
    "doc/source/_templates/README.md",
    "examples/README.md",
    "docker/Dockerfile",
    "docker/README.md",
    ".flake8",
    ".gitignore",
    "LICENSE",
    ".pre-commit-config.yaml",
    "pyproject.toml",
    "README.rst",
    "requirements/requirements_build.txt",
    "requirements/requirements_doc.txt",
    "requirements/requirements_tests.txt",
    "setup.py",
    "src/__init__.py",
    "src/_version.py",
    "src/server.py",
    "src/models/__init__.py",
    "src/observability/logger.py",
    "tests/test_metadata.py",
    "tests/test_server.py",
    "tests/conftest.py",
    "tox.ini",
    "docker-compose.yml",
]
"""A list holding all desired files to be included in the project."""


def main():
    """Entry point of the script."""
    # Get baked project location path
    project_path = Path(os.getcwd())

    # Move all requirements files into a requirements/ directory
    os.mkdir(project_path / "requirements")
    requirements_files = [
        f"requirements_{name}.txt" for name in ["build", "doc", "tests"]
    ]
    for file in requirements_files:
        shutil.move(str(project_path / file), str(project_path / "requirements"))

    # Apply isort with desired config
    isort_config = isort.settings.Config(
        line_length="{{ cookiecutter.__max_linelength }}",
        profile="black",
    )
    filepaths_list = [
        project_path / "doc/source/conf.py",
    ]
    for filepath in filepaths_list:
        isort.api.sort_file(filepath, isort_config)

    # Remove ci/cd non-desired  files
    ci_cd = "{{ cookiecutter.ci_cd_platform }}"
    if ci_cd == 'GitHub':
        DESIRED_STRUCTURE.append(".github/workflows/ci_cd.yml")
    if ci_cd == 'Azure DevOps':
        DESIRED_STRUCTURE.append("azure-pipeline.yml")

    # Remove docker non desired files
    enable_docker = "{{ cookiecutter.enable_docker }}"
    if enable_docker == 'Yes':
        DESIRED_STRUCTURE.append("docker/Dockerfile")
        DESIRED_STRUCTURE.append("docker/README.md")

    # Remove non-desired files
    keep_files(DESIRED_STRUCTURE)


if __name__ == "__main__":
    main()



"""A collection of routines focused on testing."""

import os

from ansys.templates.utils import bake_template


def assert_template_baking_process(template_path, output_path, cookiecutter_vars):
    """
    Assert if template renders properly.

    Parameters
    ----------
    template_path : ~pathlib.Path
        Path to the project template.
    output_path : ~pathlib.Path
        Path to the output baked project.
    cookiecutter_vars : dict
        A dictionary holding cookiecutter variables and their values.

    """
    bake_template(
        template_path,
        output_path,
        overwrite_if_exists=True,
        no_input=True,
        extra_context=cookiecutter_vars,
    )


def assert_file_in_baked_project(file, project_path):
    """
    Assert if file is exists inside desired output path.

    Parameters
    ----------
    file : str
        Expected file path relative to the output project path.
    project_path : ~pathlib.Path
        Path to the output project path.
    """
    assert (project_path.joinpath(file)).is_file()


def assert_files_in_baked_project(files_list, project_path):
    """
    Assert if given files exists inside desired output path.

    Parameters
    ----------
    files_list : list
        A list of expected files path relative to the output project path.
    project_path : ~pathlib.Path
        Path to the output project path.
    """
    for file in files_list:
        assert_file_in_baked_project(file, project_path)

def assert_project_structure(expected_structure, project_path):
    """Assert if project has desired structure.

    If any additional files are encountered in the rendered project, it will
    raise an AssertionError.

    Parameters
    ----------
    expected_structure : list
        A list of expected files path relative to the output project path.
    project_path : ~pathlib.Path
        Path to the output project path.

    """
    # Fix path name according to OS flavor
    separator, new_separator = ("/", "\\") if os.name != "posix" else ("/", "/")

    # Sort expected and current structures
    expected_structure = sorted(
        [file.replace(separator, new_separator) for file in expected_structure]
    )
    current_structure = sorted(
        [
            str(file.relative_to(project_path)).replace(separator, new_separator)
            for file in project_path.glob("**/*") if file.is_file()
        ]
    )

    try:
        for current_file, expected_file in zip(current_structure, expected_structure):
            assert current_file == expected_file

        assert len(current_structure) == len(expected_structure)
    except AssertionError:
        msg = f"File {current_file} not equals to {expected_file}\n\n"
        msg += f"Current structure = {current_structure}\n"
        msg += f"Expected structure = {expected_structure}\n"
        raise AssertionError(msg)



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}
import pytest


@pytest.fixture(scope="module")
def grpc_add_to_server():
    from src.stubs.pingserver_pb2_grpc import add_PingerServicer_to_server

    return add_PingerServicer_to_server


@pytest.fixture(scope="module")
def grpc_servicer():
    from src.services.pinger import Pinger

    return Pinger()


@pytest.fixture(scope="module")
def grpc_stub_cls(grpc_channel):
    from src.stubs.pingserver_pb2_grpc import PingerStub

    return PingerStub



"""Sphinx documentation configuration file."""
from datetime import datetime

from {{cookiecutter.__pkg_namespace}} import __version__
from pyansys_sphinx_theme import pyansys_logo_black

# Project information
project = "{{ cookiecutter.__pkg_name }}"
copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS, Inc."
release = version = __version__

# use the default pyansys logo
html_logo = pyansys_logo_black
html_theme = "pyansys_sphinx_theme"

html_short_title = html_title = "{{ cookiecutter.__pkg_name }}"

# specify the location of your github repo
html_theme_options = {
    "github_url": "{{ cookiecutter.__repository_url }}",
    "show_prev_next": False,
    "show_breadcrumbs": True,
    "additional_breadcrumbs": [
        ("PyAnsys", "https://docs.pyansys.com/"),
    ],
}

# Sphinx extensions
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "numpydoc",
    "sphinx.ext.intersphinx",
    "sphinx_copybutton",
]

# Intersphinx mapping
intersphinx_mapping = {
    "python": ("https://docs.python.org/dev", None),
    # kept here as an example
    # "scipy": ("https://docs.scipy.org/doc/scipy/reference", None),
    # "numpy": ("https://numpy.org/devdocs", None),
    # "matplotlib": ("https://matplotlib.org/stable", None),
    # "pandas": ("https://pandas.pydata.org/pandas-docs/stable", None),
    # "pyvista": ("https://docs.pyvista.org/", None),
    # "grpc": ("https://grpc.github.io/grpc/python/", None),
}

# numpydoc configuration
numpydoc_show_class_members = False
numpydoc_xref_param_type = True

# Consider enabling numpydoc validation. See:
# https://numpydoc.readthedocs.io/en/latest/validation.html#
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    "GL08",  # The object does not have a docstring
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    "SS02",  # Summary does not start with a capital letter
    # "SS03", # Summary does not end with a period
    "SS04",  # Summary contains heading whitespaces
    # "SS05", # Summary must start with infinitive verb, not third person
    "RT02",  # The first line of the Returns section should contain only the
    # type, unless multiple values are being returned"
}


# static path
html_static_path = ["_static"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"



"""
{{ cookiecutter.product_name }}.

{{ cookiecutter.library_name }}
"""

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__.replace(".", "-"))
